/*
** errno.h
**
** Emulation of the Unix errno.h header file for PRIMOS
**
** Author: Peter Eriksson <pen@lysator.liu.se>
*/

#ifndef __ERRNO_H__
#define __ERRNO_H__

#include <errd.h>

#define ENOENT e$fntf

#endif

