/*
** stdlib.h
**
** Emulation of the Unix stdlib.h header file for PRIMOS
**
** Author: Peter Eriksson <pen@lysator.liu.se>
*/

#ifndef __STDLIB_H__
#define __STDLIB_H__

extern char *malloc();
extern char *calloc();

#endif

