/*
** fcntl.h
**
** Emulation of the Unix fcntl.h header file for PRIMOS
**
** Author: Peter Eriksson <pen@lysator.liu.se>
*/

#ifndef __FCNTL_H__
#define __FCNTL_H__

#define O_RDONLY 0
#define O_WRONLY 1
#define O_RDWR   2

#define O_BINARY 0
#define O_EXCL   0
#define O_NDELAY 0
#define O_CREAT  0
#define O_TRUNC  0

#endif
